/********************************************************/
/* Program         : color.c 							*/
/* Function        : color library for social jewlery	*/
/* Author          : Chris Coulston     				*/
/* Platform        : HiTech (Lite Mode)  V9.71a			*/
/* Target          : PIC16LF1825 						*/
/*    			   : 14-pin SMT version					*/
/********************************************************/
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <htc.h>
#include <pic16lf1825.h>
#include "iColor.h"

/********************************************************/
/*					TX = WriteByte						*/
/* Send a start bit followed by the data bits starting	*/
/* from the most to least significant.					*/
/* Don't reinitialize TMR0 back to 0 after each action	*/
/* so that the total transmit time does not get skewed 	*/
/* by the data bits contined in the message.			*/
/*														*/
/*	low		high		Meaning							*/
/*	1u		1u			Logic 0							*/ 
/*	1u		2u			Logic 1							*/
/*	3u		3u			Start							*/
/*	1u		2u			Stop							*/
/********************************************************/
#define HalfUnit		0xE0
#define OneUnit			0xC0
#define TwoUnit			0x80

void TX(unsigned char send) {

	unsigned char i, temp;
	temp = send;

	// Create a start bit, 3 units low followed by 3 units high
	RA2 = 1;	TMR0 = OneUnit;				TMR0IF = 0;		while(!TMR0IF);		// 3 units of light on, produces logic 0 at V photo
	RA2 = 1;	TMR0 += TwoUnit;			TMR0IF = 0;		while(!TMR0IF);		// 3 units of light on, produces logic 0 at V photo

	RA2 = 0;	TMR0 += OneUnit;			TMR0IF = 0;		while(!TMR0IF);		// 3 units of light off produces logic 1 at V photo
	RA2 = 0;	TMR0 += TwoUnit;			TMR0IF = 0;		while(!TMR0IF);		// 3 units of light off produces logic 1 at V photo

	// Now transmit the data bits, MSbit first
	for (i=0; i<8; i++) {
		RA2 = 1;	TMR0 += OneUnit;	TMR0IF = 0;		while(!TMR0IF);			// Always preface bit with 1 unit of light
		RA2 = 0;	TMR0 += OneUnit;	TMR0IF = 0;		while(!TMR0IF);			// followed by at least 1 unit of no light
		if ((temp & 0x80) == 0x80) {
					TMR0 += OneUnit;	TMR0IF = 0;		while(!TMR0IF);			// A logic 1 gets an additional 1 unit of light off
		} // end if
		temp = temp << 1;		
	} // end index each bit of transmitted bit

	// Send the stop bit
	RA2 = 1;	TMR0 += TwoUnit;		TMR0IF = 0;		while(!TMR0IF);			// TMR0 rollover for 1 unit
	RA2 = 0;	TMR0 += TwoUnit;		TMR0IF = 0;		while(!TMR0IF);			// TMR0 rollover for 1 unit

} 


/********************************************************/
/* This is really just a testing piece of code so that	*/
/* the user can change the speed of the light port from	*/
/* a centrally controlled menu and determine the max	*/
/* reliable speed of the port.							*/
/********************************************************/
void SetTimer0Pre(void) {

	switch(pre) {

		case 7:
			PS2 = 1;	PS1 = 1;	PS0 = 1;	// 32:1 prescalar
			break;

		case 6:
			PS2 = 1;	PS1 = 1;	PS0 = 0;	// 32:1 prescalar
			break;

		case 5:
			PS2 = 1;	PS1 = 0;	PS0 = 1;	// 32:1 prescalar
			break;

		case 4:
			PS2 = 1;	PS1 = 0;	PS0 = 0;	// 32:1 prescalar
			break;

		case 3:
			PS2 = 0;	PS1 = 1;	PS0 = 1;	// 16:1 prescalar
			break;

		case 2:
			PS2 = 0;	PS1 = 1;	PS0 = 0;	// 8:1 prescalar
			break;

		case 1:
			PS2 = 0;	PS1 = 0;	PS0 = 1;	// 4:1 prescalar
			break;

		case 0:
			PS2 = 0;	PS1 = 0;	PS0 = 0;	// 2:1 prescalar
			break;

		default:
			break;

}	} // end SetTimer0Pre



/********************************************************/
/*				DigitalRX = ReadByte					*/
/* This receive uses the logic thresholding to 			*/
/* determine the value of the photo voltage				*/
/* RA2 > Vih reads logic 1		Vih=0.25*Vdd+0.8 = 1.7	*/
/* RA2 < Vil reads logic 0		Vil=0.15*Vdd = 0.54		*/
/*														*/
/* Its important to make sure that the longest bit		*/
/* length is less than 1 TMR0 rollover.  If, while		*/
/* reading we get a period of inactivity longer than	*/
/* 1 TMR0 rollover (1.024 mS) then we know something is	*/
/* wrong with the transmission and to abort the current	*/
/* read operation.										*/
/*														*/
/* Added a digital low pass filter to remove the effect	*/
/* of spurious noise on the decoding of the pulses.		*/
/*														*/
/* return NULL if there is a problem with the start bit	*/
/*														*/
/* When you are receiving the second and beyond char	*/
/* its possible that the subroutine overhead cuts into	*/
/* the TX of the next byte, so we compensate with a 	*/
/* fudge factor that depends if we are examining the	*/
/* first character or the reset.  This is the reason	*/
/* for the argument to the procedure					*/
/********************************************************/
#define		MASK	0x03		// Defines the number of consectutive reads
								// it takes for the V photo to conviencingly 
								// be considered to be changed from its prior state.

unsigned char DigitalRX(unsigned char process) {

	unsigned char incomming, i, low, high, sample, num_reads;

	//sprintf(string,"Digital RX.\r\n");									Myprintf(string);
			
	for (i=0; i<20; i++) array[i] = 0x00;
	sample = 0xFF;

	// Blow by the idle time with no light
	while((sample & MASK) != 0x00) sample = (sample << 1) | RA4;

	num_reads = 9;
	if (process == FALSE) num_reads = 8;

	for (i=0; i<num_reads; i++) {					// Read all 8 data bits starting with MSB

		// Measure the time RA4 is logic low
		TMR0 = 0x00;	TMR0IF = 0;			// clear the timer and flag so that we can detect a time-out
		while (((sample & MASK) != MASK) && (!TMR0IF))	// Time the pulse low and give yourself an out if it takes too long
			sample = (sample << 1) | RA4;	// remove photo voltage noise using a low pass filter
		if (TMR0IF) low = 0xFF;				// Something is amiss, denote this with timer value and just abort this read attempt	
		else low = TMR0;					// Record the time low

		// Measure the time that RA4 is logic 1
		TMR0 = 0x00;	TMR0IF = 0;			// Start timer over again
		while (((sample & MASK)!= 0x00) && (!TMR0IF))	// Record time with light off while giving yourself an out
			sample = (sample << 1) | RA4;	// remove photo voltage noise using a low pass filter
		if (TMR0IF)	high = 0xFF;			// Something is amiss,denote this with the timer value and just abort this read attempt
		else high = TMR0;					// Record the time low

		array[i<<1]=low;
		array[(i<<1)+1]=high;		// Stored for diagonistics

		if ( (low == 0xFF) || (high == 0xFF)) break;

	} // end for

	// Blow through the stop bit when its at logic 0
	TMR0 = 0x00;	TMR0IF = 0;			// clear the timer and flag so that we can detect a time-out
	while (((sample & MASK) != MASK) && (!TMR0IF))	// Time the pulse low and give yourself an out if it takes too long
		sample = (sample << 1) | RA4;	// remove photo voltage noise using a low pass filter
	if (TMR0IF) low = 0xFF;				// Something is amiss, denote this with timer value and just abort this read attempt	
	else low = TMR0;					// Record the time low
	array[18]=low;
	array[19]=0xFF;

	if (process == FALSE) return(0);

	// The heuristic for decoding is that a valid start bit means valid data bits
	if (DigitalClassifyBit(array[0],array[1]) == START) {
		incomming = 0;
		for (i=1; i<9; i++) {
			incomming = incomming << 1;
			sample = DigitalClassifyBit(array[i<<1],array[(i<<1)+1]);
			if (sample == LOGIC1)	incomming = incomming | 0x01;
			if (sample == BAD_DATA)	{
				sprintf(string,"Data bit %d\r\n",i);						Myprintf(string);
				return(0);
		}	}
		return(incomming);
	} else {
		sprintf(string,"Bad Start Bit.\r\n");									Myprintf(string);	
		return(0);
	}
} // end DigitalRX


/********************************************************/
/*				AnalogClassifyBit						*/
/* A bit in this system will be composed of a time low 	*/
/* and a time high.  This is borrowed from IR remote	*/
/* control protocol.									*/
/*	low		high		Meaning							*/
/*	1mS		1mS			Logic 0							*/ 
/*	1mS		2mS			Logic 1							*/
/*	3mS		3mS			Start							*/
/********************************************************/
//#define CompOneUnit			(0xFF - OneUnit + 1)
//#define CompTwoUnit			(0xFF - TwoUnit + 1)
//#define CompThrUnit			(0xFF - ThrUnit + 1)
//#define CompHalfUnit		(0xFF - HalfUnit + 1)

#define CompOneUnit			64
#define CompTwoUnit			128
#define CompThrUnit			192
#define CompHalfUnit		40

unsigned char DigitalClassifyBit(unsigned char low, unsigned char high) {

	if (ALMOST(low,CompOneUnit,CompHalfUnit)) {
		if (ALMOST(high,CompOneUnit,CompHalfUnit)) return(LOGIC0);
		else if (ALMOST(high,CompTwoUnit,CompHalfUnit)) return(LOGIC1);
		else {
				// sprintf(string,"Bad Data low=%d high=%d CompThrUnit=%d CompHalfUnit=%d\r\n", low, high, CompOneUnit, CompHalfUnit);							Myprintf(string);
				return(BAD_DATA);
	} 	} //end data bit

	if (ALMOST(low,CompThrUnit,CompHalfUnit) && ALMOST(high,CompThrUnit,CompHalfUnit)) return(START);

	// sprintf(string,"Bad Start low=%d high=%d CompThrUnit=%d CompHalfUnit=%d\r\n", low, high, CompThrUnit, CompHalfUnit);							Myprintf(string);
	return(BAD_START);

} // end ClassifyChar


/********************************************************/
/********************************************************/
unsigned char ALMOST(unsigned char actual, unsigned char time, unsigned char delta) {

	if ( (actual > (time - delta)) && (actual < (time + delta))) return (TRUE);
	else return(FALSE);

} // end ALMOST


/********************************************************/
/*				DigitalRXSense 							*/
/* This function will record all the relevant data		*/
/* regarding the incoming waveform when measures using	*/
/* the digital technique on RA4.  The sampling rate on	*/
/* The digital channel will be 200uS, so in order to	*/
/* sample the entire waveform we will need an array of	*/
/* of at most 3*10 = 30 sample values.					*/
/********************************************************/
void DigitalRXArray(unsigned char *array) {

	unsigned char samples, i, j;

	// Blow by some idle time
	while(RA4 == 1);

	TMR0 = SampleInterval;				// create a delay with 
	TMR0IF = 0;	
	for (j=0; j<SampleLength; j++) {
		samples = 0;
		for (i=0; i< 8; i++) {
			while(!TMR0IF);		// TMR0 rollover
			samples = (samples << 1) | RA4;
			TMR0 += SampleInterval;				// create a delay with 
			TMR0IF = 0;	
		} // end for i
		array[j] = samples;	
	} // end receive characters

} // end DigitalRX





/********************************************************/
/*				AnalogRX = AnalogReadByte				*/
/* Read a Byte using the Analog to Digital Converter	*/
/********************************************************/
unsigned char AnalogRX(void) {

	unsigned char samples, i, j;

	GIE = 0;							// Shit just got real, were gonna go all foreground on this
	// Turn off the LEDs
	CCP3CON = 0x00;		RA2 = 0;		// Turn PWM off and pin low
	CCP4CON = 0x00;		RC1 = 0;		// Turn PWM off and pin low
	CCP2CON = 0x00;		RC3 = 0;		// Turn PWM off and pin low
	RA5 = 1;							// Switch on the sensing circuit after disabling PW
	// Wait for the logic low on the Photo line
	do {
		if (RCIF) break;								
		ADGO = 1;							// tell you what the red LED sees
		while(ADGO);
	} while (ADRESH > ((DACCON1 & 0x1F) << 2));

	for (j=0; j<SampleLength; j++) {
		samples = 0;
		for (i=0; i< 8; i++) {
			while(!TMR0IF);		// TMR0 rollover
			ADGO = 1;							// tell you what the red LED sees
			while(ADGO);
			samples = samples << 1;
			if (ADRESH > ((DACCON1 & 0x1F) << 2))	samples |= 1;
		} // end for i
		array[j] = samples;	
	} // end receive characters

	RA5 = 0;							// Turn sensing off prior to enabling PWM
	TMR2 = 0xF8;
	CCP3CON = 0x0C;						// Turn PWM back on
	CCP4CON = 0x0C;						// Turn PWM back on
	CCP2CON = 0x0C;						// Turn PWM back on
	i = RCREG;
	GIE = 1;							// Turn back on interrupts
	
	return(i);

} // end RX


/********************************************************/
/*				AnalogReadBit							*/
/* The assumption is that the RX line is at logic		*/
/* 0 and that we are in the midst of a sequence			*/
/* The place where the ADC is used to read the V photo	*/
/* line and convert the duration into a bit value		*/
/********************************************************/
unsigned char AnalogReadBit(void) {

	unsigned char low = 0;
	unsigned char high = 0;

		do {
			if (RCIF) break;
			ADGO = 1;							// tell you what the red LED sees
			while(ADGO);
			low += 1; 					// Count the number of cycles high
		} while (ADRESH < ((DACCON1 & 0x1F) << 2));

		do {
			if (RCIF) break;
			ADGO = 1;							// tell you what the red LED sees
			while(ADGO);
			high += 1;					// Count the number of cycles low
		} while (ADRESH > ((DACCON1 & 0x1F) << 2));	

		return(AnalogClassifyBit(low, high));

} // end ReadBit


/********************************************************/
/*				AnalogClassifyBit						*/
/* A bit in this system will be composed of a time low 	*/
/* and a time high.  This is borrowed from IR remote	*/
/* control protocol.									*/
/*	low		high		Meaning							*/
/*	1mS		1mS			Logic 0							*/ 
/*	1mS		2mS			Logic 1							*/
/*	3mS		3mS			Start							*/
/********************************************************/
#define ANALOGHALFMILLISECOND		7
#define ANALOGONEMILLISECOND		15
#define ANALOGTWOMILLISECOND		30
#define ANALOGTHREEMILLISECOND	45
unsigned char AnalogClassifyBit(unsigned char low, unsigned char high) {

	if (ALMOST(low,ANALOGONEMILLISECOND,ANALOGHALFMILLISECOND)) {
		if (ALMOST(high,ANALOGONEMILLISECOND,ANALOGHALFMILLISECOND)) return(LOGIC0);
		else if (ALMOST(high,ANALOGTWOMILLISECOND,ANALOGHALFMILLISECOND)) return(LOGIC1);
		else return(BAD_DATA);
	} // end data bit

	if (ALMOST(low,ANALOGTHREEMILLISECOND,ANALOGHALFMILLISECOND) && ALMOST(high,ANALOGTHREEMILLISECOND,ANALOGHALFMILLISECOND)) return(START);

	return(BAD_START);

} // end ClassifyChar



/********************************************************/
/*					JewleryRX							*/
// This tour is based on the 24-color circle with the exception that
// some of the intermediate colors have been removed to cut down on
// the size of the array.  The program does a simple form
// of linar interpolation using a pseudo-fixed point 
// method where the 256 steps between each array point
// are incremented by a "fraction" which adds into the 
// 8 most significant bits.
// order is:					red		green 	blue
//								CCPR3L	CCPR4L	CCPR2L
//
// The ISR will constantly look at the level of RED
// illumination and store it in an array.  Every time
// that the array is filled up, the 
/********************************************************/
#define MAX		0x80
#define HALF	0x40
void JewleryRX(void) {

	unsigned char step, i, j, temp, avg, old_avg, trigger, first, low, sample, incomming, enable_sense;
	signed int red, green, blue;
	signed int r0, g0, b0, r1, g1, b1, dr, dg, db;
	unsigned int	bigsum, delta;

	unsigned char tour[13][3] = {		{MAX ,0x00,0x00},		// red
										{MAX ,HALF,0x00},		// orange
										{MAX ,MAX ,0x00},		// yellow
										{HALF,MAX ,0x00},		// chartreuse
										{0x00,MAX ,0x00},		// green
										{0x00,MAX ,HALF},		// spring green
										{0x00,MAX ,MAX },		// cyan 
										{0x00,HALF,MAX },		// azure
										{0x00,0x00,MAX },		// blue
										{HALF,0x00,MAX },		// violet
										{MAX ,0x00,MAX },		// magenta
										{MAX ,0x00,HALF},		// rose
										{MAX ,0x00,0x00}};		// red

	new_photo_set = FALSE;
	enable_sense = 0;
	first = TRUE;
	trigger = FALSE;
	sprintf(string,"Interactive Jewlery: Hit a key to exit.\r\n");					Myprintf(string);
	i = 0;

	while(!RCIF){

		dr = tour[i+1][0] - tour[i][0];
		dg = tour[i+1][1] - tour[i][1];
		db = tour[i+1][2] - tour[i][2];

		red = tour[i][0]<<8;			
		green = tour[i][1]<<8;
		blue = tour[i][2]<<8;

		for (step=0; step < 255; step ++) {

			red += dr;
			green += dg;
			blue += db;

			CCPR3L = (red >> 8)&0xFF;
			CCPR4L = (green >> 8)&0xFF;
			CCPR2L = (blue >> 8)&0xFF;

			// Every time that we get a new photo set check if there is strong variation in
			// the photo intensity and deviation.  This indicates a periodic source nearby.
			// These calculations take time so compensate the delay accordingly.  Only do this
			// after a suitable delay after reading the incoming data stream
			if ((new_photo_set == TRUE) && (enable_sense > 9)) {			

				// Now perform average and deviation on the photo data obtained
				new_photo_set = FALSE;
				bigsum = 0;
				for (j=0; j< PhotoArrayLength; j++) bigsum += photo_array[j];
				avg = bigsum >> LogPhotoArrayLength;
//				delta = 0;
//				for (j=0; j< PhotoArrayLength; j++) {
//					if (avg > photo_array[j])	delta += (avg - photo_array[j]);
//					else						delta += (photo_array[j] - avg);
//				}

				if (avg > old_avg)	temp = avg - old_avg;
				else				temp = old_avg - avg;

				if ((temp > 30) && (first == FALSE)) trigger = TRUE;
				first = FALSE;
				old_avg = avg;

				// The above calculations take about 0.5mS so compensate the delay accordingly
				TMR0 = 0x80;						// create a 1.5mS delay with 2 rollovers
				TMR0IF = 0;		while(!TMR0IF);		// 1.5ms when complete

			} else {	

				// Hold the illumination pattern for some number of iterations, either speeds-up or
				// slows-down the color cycle.  Could be used to indicate the presence of another 
				// pendant by speeding up the cycle and making the pendant look "excited"

				TMR0 = 0x00;						// create a 2mS delay with 2 rollovers
				TMR0IF = 0;		while(!TMR0IF);		// 1mS when complete
				TMR0IF = 0;		while(!TMR0IF);		// 2ms when complete
	
			} // end what kind of delay

			// If there is no delay control, the pendant will continously enter into the sensing mode
			// to read the incoming data stream and never get on with the task of generating a color
			// sequence.  The variable "enable_sense" is reset back to 0 during the reading phase and
			// incremented in the color sequencing index below.  This allows the pendant to do at least
			// some color display between bouts of reading data streams.
			if ((trigger == TRUE) && (enable_sense > 9))  break;

		} // end for a step in the color transition between entry i and i+1


		if ((trigger == TRUE) && (enable_sense > 9)) {						// Turn off all display and background processing so that we can look at the incoming bit pattern
			trigger = FALSE;												// Reset flag for next attempt
			enable_sense = 0;												// reset counter 
			first = TRUE;													// You going to need to start resampling the sensor
			RunSilent();

			TMR1H = 0x00;
			TMR1L = 0x00;
			TMR1IF = 0;
			sample = 0xFF;

			// A search commencemeces for the begining of message character "S".  This is done in stages
			// Stage 1 - look for a start-bit
			// Stage 2 - look for the character "S"
			while(!TMR1IF) {

				// Burn through any interval which is logic 1
				while (((sample & MASK)!= 0x00) && (!TMR1IF))	// Burn through an interval which is logic 1, waiting for
					sample = (sample << 1) | RA4;				// a time interval which is logic 0, possibly a start-bit

				// Measure the time that RA4 is logic 0
				TMR0 = 0x00;	TMR0IF = 0;						// Start timer at zero to measure time
				while (((sample & MASK)!= MASK) && (!TMR0IF))	// Record time with light off while giving yourself an out
					sample = (sample << 1) | RA4;				// remove photo voltage noise using a low pass filter
				if (TMR0IF)	low = 0xFF;							// Something is amiss,denote this with the timer value and just abort this read attempt
				else low = TMR0;								// Record the time low

				// If we have found a start-bit, read the rest of the character
				if (ALMOST(low,CompThrUnit,CompHalfUnit)) {		// The start condition is three units of time low

					DigitalRX(FALSE);							// read the character without processing (because the start-bit is not included)

					// Now parse the received character
					incomming = 0;
					for (i=0; i<8; i++) {
						incomming = incomming << 1;
						sample = DigitalClassifyBit(array[i<<1],array[(i<<1)+1]);
						if (sample == LOGIC1)	incomming = incomming | 0x01;
						if (sample == BAD_DATA)	{
							sprintf(string,"Data bit %d\r\n",i);						Myprintf(string);
							// exit(0);
					}	}

					// Wait until you get to the end of the message, denoted with a carriage return
					while(incomming != '\n') incomming = DigitalRX(TRUE);
					
					// Wait until you get to the end of the message, denoted with a carriage return
					i=0;
					do {
						string[i++] = DigitalRX(TRUE);
					} while(string[i-1] != '\n');

					// Myprintf(string);
					TMR1H = 0xFF;				// Cause a rollover which will set the flag and allow
					TMR1L = 0xFE;				// the program to leave the sensing loop controled by TMR1IF

					switch (string[1]) {
						case 'r':
							i = 11;				// i will be incremented below and roll over to 0 which is red's index
							break;
						case 'g':
							i = 3;				// i will be incremented below to 4 which is green
							break;
						case 'b':
							i = 7;				// i will be incremented below to 8 which is blue
							break;
						default:
							i = 1;				// Error in reception will be yellow which is easy to see
							break;
					} // end which color will the cycle pick-up at	
				} // end while having found a start-bit
			} // end looking for a start-bit, while(!TMR1IF)

			RunActive();

		} // end if sensed a change in red, if (triggger == TRUE)

		i = i + 1;							// Increment index into the Color Look-up Table
		if (i == 12) i = 0;
		enable_sense += 1;
		if (enable_sense > 10) enable_sense = 10;

	} // end while because someone hit a key to end the Jewerly RX

	CCPR3L = 0x00;	CCPR4L = 0x00;	CCPR2L = 0x00;			// Off
	i = RCREG;
	sprintf(string,"All done\r\n");					Myprintf(string);

} // end JewleryRX



/********************************************************/
/*					RunActive							*/
/********************************************************/
void RunActive(void) {

	ANSA4 = 1;							// Make the photovoltage a analog input
	RA5 = 0;							// Turn sensing off prior to enabling PWM
	TMR2 = 0xF8;
	CCP3CON = 0x0C;						// Turn PWM back on
	CCP4CON = 0x0C;						// Turn PWM back on
	CCP2CON = 0x0C;						// Turn PWM back on
	ADON = 1;							// Turn ADC back on
	GIE = 1;							// Turn back on interrupts	
	PSA = 1;							// Turn off TMR0 prescalar

} 

/********************************************************/
/*					RunSilent							*/
/********************************************************/
void RunSilent(void) {

	GIE = 0;							// Shit just got real, were gonna go all foreground on this
	ADON = 0;							// Turn off the ADC and sense in digital mode
	PSA = 0;							// assign a prescalar to TMR0
	SetTimer0Pre();	

	// Turn off the LEDs
	CCP3CON = 0x00;		RA2 = 0;		// Turn PWM off and pin low
	CCP4CON = 0x00;		RC1 = 0;		// Turn PWM off and pin low
	CCP2CON = 0x00;		RC3 = 0;		// Turn PWM off and pin low
	RA5 = 1;							// Switch on the sensing circuit after disabling PWM
	ANSA4 = 0;							// Make the photovoltage a digital input

} // end RunSilent


/********************************************************/
/*					JewleryTX							*/
/********************************************************/

void JewleryTX(void) {

	unsigned char i;

	sprintf(string,"\r\n"); 														Myprintf(string);
	sprintf(string,"\r\n------------------------------\r\n");		 				Myprintf(string);
	sprintf(string,"r: red\r\n");													Myprintf(string);
	sprintf(string,"g: green\r\n");													Myprintf(string);
	sprintf(string,"b: blue\r\n");													Myprintf(string);
	sprintf(string,"Select a color> ");												Myprintf(string);

	while(!RCIF);	/* set when register is not empty */

	switch(RCREG) {
		case 'r':
			sprintf(string,"Sred\r\n");
			break;
		case 'g':
			sprintf(string,"Sgre\r\n");
			break;
		case 'b':
			sprintf(string,"Sblu\r\n");
			break;
	} // end switch

	RunSilent();
	while(!RCIF) for (i=0; i<strlen(string); i++) TX(string[i]);
	RunActive();

	i = RCREG;

} // end JewleryTX


/********************************************************/
/********************************************************/
void Synch(void) {

	unsigned char sample = 0xFF;
	unsigned char time;

	RunSilent();	

	// Blow by the idle time with no light, logic high on RA4
	while((sample & MASK) != 0x00) sample = (sample << 1) | RA4;

	// Blow by time with light illuminating the red LED
	while((sample & MASK) != MASK) sample = (sample << 1) | RA4;

	while (!RCIF) {

		// We are looking for the negative edge on RA4
		while((sample & MASK) != 0x00) sample = (sample << 1) | RA4;

		TMR0 = 0x00;	TMR0IF = 0;			// Start timer over again
		while (((sample & MASK)!= MASK) && (!TMR0IF))	// Record time with light off while giving yourself an out
			sample = (sample << 1) | RA4;	// remove photo voltage noise using a low pass filter

		time = TMR0;							// The TX unit should be "off" right now
		if (!TMR0IF) {							// If we measured a pulse then reproduce it
			RA5 = 0;							// Turn sensing off prior to outputing a voltage on RA2
			RA2 = 1;							// Turn on red LED for the same amount of time
			TMR0 = 0x00;	TMR0IF = 0;			//	that we read the pulse on RA4
			while(TMR0 < time);					//  *tapping feet*
			RA2 = 0;							// Turn off the red LED and immediatly go back to 
		} else RA2 = 0;							// looking for the falling edge on RA4

		RA5 = 1;								// Turn on sensing circuit so that we can measure the next pulse

		TMR0 = 0xE0;	TMR0IF = 0;				// A little recovery time for the sensing LED to recover
		while(!TMR0IF);

		//sprintf(string,"%x\r\n",time);					Myprintf(string);

	} // end for

	RunActive();

} // end Synch



/********************************************************/
/*					ColorCycle							*/
// This tour is based on the 24-color circle with the exception that
// some of the intermediate colors have been removed to cut down on
// the size of the array.  The program does a simple form
// of linar interpolation using a pseudo-fixed point 
// method where the 256 steps between each array point
// are incremented by a "fraction" which adds into the 
// 8 most significant bits.
// order is:					red		green 	blue
//								CCPR3L	CCPR4L	CCPR2L
/********************************************************/
#define MAX		0x80
#define HALF	0x40
void ColorCycle(void) {

	unsigned char step, i = 0;
	signed int red, green, blue;
	signed int r0, g0, b0, r1, g1, b1, dr, dg, db;
	// unsigned char	string[64];


	unsigned char tour[13][3] = {	{MAX ,0x00,0x00},		// red
									{MAX ,HALF,0x00},		// orange
									{MAX ,MAX ,0x00},		// yellow
									{HALF,MAX ,0x00},		// chartreuse
									{0x00,MAX ,0x00},		// green
									{0x00,MAX ,HALF},		// spring green
									{0x00,MAX ,MAX },		// cyan 
									{0x00,HALF,MAX },		// azure
									{0x00,0x00,MAX },		// blue
									{HALF,0x00,MAX },		// violet
									{MAX ,0x00,MAX },		// magenta
									{MAX ,0x00,HALF},		// rose
									{MAX ,0x00,0x00}};		// red

	sprintf(string,"Color list: Hit a key to exit.\r\n");					Myprintf(string);

	while(!RCIF){

		dr = tour[i+1][0] - tour[i][0];
		dg = tour[i+1][1] - tour[i][1];
		db = tour[i+1][2] - tour[i][2];

		red = tour[i][0]<<8;			
		green = tour[i][1]<<8;
		blue = tour[i][2]<<8;

		//ColorSat(red, green, blue);

		for (step=0; step < 255; step ++) {

			red += dr;
			green += dg;
			blue += db;

			CCPR3L = red >> 8;
			CCPR4L = green >> 8;
			CCPR2L = blue >> 8;

			TMR0 = 0x00;						// create a 2mS delay with 2 rollovers
			TMR0IF = 0;		while(!TMR0IF);		// 1mS when complete
			TMR0IF = 0;		while(!TMR0IF);		// 2ms when complete
				
		} // step

		i = i + 1;
		if (i == 12) i = 0;

	} // end while
	CCPR3L = 0x00;	CCPR4L = 0x00;	CCPR2L = 0x00;			// Off
	i = RCREG;
	sprintf(string,"All done\r\n");					Myprintf(string);

} // end ColorCycle


/********************************************************/
/*					ColorSat							*/
// Moves the color cordinate from that given by the 
// parameters to white. The progression of the convergence is
// geometric so a proportional amount of the difference
// between the current color and max saturation is added
// to the current color at each iteration.
/********************************************************/
void ColorSat(unsigned char red, unsigned char green, unsigned char blue) {

	unsigned char i;
	unsigned int curr_r, curr_g, curr_b;

	curr_r = red << 8;
	curr_g = green << 8;
	curr_b = blue << 8;

	for (i=0; i<255; i++) {
		curr_r += MAX - (curr_r >> 8);	// add 6.25% at each iteration
		curr_g += MAX - (curr_g >> 8);
		curr_b += MAX - (curr_b >> 8);

		CCPR3L = (curr_r >> 8);
		CCPR4L = (curr_g >> 8);
		CCPR2L = (curr_b >> 8);

		TMR0 = 0x00;						// create a 2mS delay with 2 rollovers
		TMR0IF = 0;		while(!TMR0IF);		// 1mS when complete
		TMR0IF = 0;		while(!TMR0IF);		// 2ms when complete
			
	} // end for i

} // end ColorSat

